@@ -1,3 +1,6 @@
+require 'simplecov'
+SimpleCov.start
+
 require 'chake/node'
 require 'chake/backend'
 
